@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -54,23 +54,6 @@ typedef enum {
 } GMimeFilterBestFlags;
 
 
-/**
- * GMimeBestEncoding:
- * @GMIME_BEST_ENCODING_7BIT: The stream data must fit within the 7bit ASCII range.
- * @GMIME_BEST_ENCODING_8BIT: The stream data may have bytes with the high bit set, but no null bytes.
- * @GMIME_BEST_ENCODING_BINARY: The stream may contain any binary data.
- *
- * Used with g_mime_filter_best_encoding() as the 'required'
- * argument. These values provide a means of letting the filter know
- * what the encoding requirements are for the stream.
- **/
-typedef enum {
-	GMIME_BEST_ENCODING_7BIT,
-	GMIME_BEST_ENCODING_8BIT,
-	GMIME_BEST_ENCODING_BINARY
-} GMimeBestEncoding;
-
-
 /**
  * GMimeFilterBest:
  * @parent_object: parent #GMimeFilter
@@ -126,7 +109,7 @@ GMimeFilter *g_mime_filter_best_new (GMimeFilterBestFlags flags);
 
 const char *g_mime_filter_best_charset (GMimeFilterBest *best);
 
-GMimeContentEncoding g_mime_filter_best_encoding (GMimeFilterBest *best, GMimeBestEncoding required);
+GMimeContentEncoding g_mime_filter_best_encoding (GMimeFilterBest *best, GMimeEncodingConstraint constraint);
 
 G_END_DECLS
 
