@@ -57,16 +57,6 @@ static void g_mime_multipart_encrypted_class_init (GMimeMultipartEncryptedClass
 static void g_mime_multipart_encrypted_init (GMimeMultipartEncrypted *mps, GMimeMultipartEncryptedClass *klass);
 static void g_mime_multipart_encrypted_finalize (GObject *object);
 
-/* GMimeObject class methods */
-static void multipart_encrypted_prepend_header (GMimeObject *object, const char *header, const char *value);
-static void multipart_encrypted_append_header (GMimeObject *object, const char *header, const char *value);
-static void multipart_encrypted_set_header (GMimeObject *object, const char *header, const char *value);
-static const char *multipart_encrypted_get_header (GMimeObject *object, const char *header);
-static gboolean multipart_encrypted_remove_header (GMimeObject *object, const char *header);
-static void multipart_encrypted_set_content_type (GMimeObject *object, GMimeContentType *content_type);
-static char *multipart_encrypted_get_headers (GMimeObject *object);
-static ssize_t multipart_encrypted_write_to_stream (GMimeObject *object, GMimeStream *stream);
-
 
 static GMimeMultipartClass *parent_class = NULL;
 
@@ -99,21 +89,11 @@ g_mime_multipart_encrypted_get_type (void)
 static void
 g_mime_multipart_encrypted_class_init (GMimeMultipartEncryptedClass *klass)
 {
-	GMimeObjectClass *object_class = GMIME_OBJECT_CLASS (klass);
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	
 	parent_class = g_type_class_ref (GMIME_TYPE_MULTIPART);
 	
 	gobject_class->finalize = g_mime_multipart_encrypted_finalize;
-	
-	object_class->prepend_header = multipart_encrypted_prepend_header;
-	object_class->append_header = multipart_encrypted_append_header;
-	object_class->remove_header = multipart_encrypted_remove_header;
-	object_class->set_header = multipart_encrypted_set_header;
-	object_class->get_header = multipart_encrypted_get_header;
-	object_class->set_content_type = multipart_encrypted_set_content_type;
-	object_class->get_headers = multipart_encrypted_get_headers;
-	object_class->write_to_stream = multipart_encrypted_write_to_stream;
 }
 
 static void
@@ -137,54 +117,6 @@ g_mime_multipart_encrypted_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
-multipart_encrypted_prepend_header (GMimeObject *object, const char *header, const char *value)
-{
-	GMIME_OBJECT_CLASS (parent_class)->prepend_header (object, header, value);
-}
-
-static void
-multipart_encrypted_append_header (GMimeObject *object, const char *header, const char *value)
-{
-	GMIME_OBJECT_CLASS (parent_class)->append_header (object, header, value);
-}
-
-static void
-multipart_encrypted_set_header (GMimeObject *object, const char *header, const char *value)
-{
-	GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
-}
-
-static const char *
-multipart_encrypted_get_header (GMimeObject *object, const char *header)
-{
-	return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
-}
-
-static gboolean
-multipart_encrypted_remove_header (GMimeObject *object, const char *header)
-{
-	return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
-}
-
-static void
-multipart_encrypted_set_content_type (GMimeObject *object, GMimeContentType *content_type)
-{
-	GMIME_OBJECT_CLASS (parent_class)->set_content_type (object, content_type);
-}
-
-static char *
-multipart_encrypted_get_headers (GMimeObject *object)
-{
-	return GMIME_OBJECT_CLASS (parent_class)->get_headers (object);
-}
-
-static ssize_t
-multipart_encrypted_write_to_stream (GMimeObject *object, GMimeStream *stream)
-{
-	return GMIME_OBJECT_CLASS (parent_class)->write_to_stream (object, stream);
-}
-
 
 /**
  * g_mime_multipart_encrypted_new:
@@ -213,9 +145,10 @@ g_mime_multipart_encrypted_new (void)
  * g_mime_multipart_encrypted_encrypt:
  * @mpe: multipart/encrypted object
  * @content: MIME part to encrypt
- * @ctx: encryption cipher context
+ * @ctx: encryption crypto context
  * @sign: %TRUE if the content should also be signed or %FALSE otherwise
  * @userid: user id to use for signing (only used if @sign is %TRUE)
+ * @hash: digest algorithm to use when signing
  * @recipients: an array of recipients to encrypt to
  * @err: a #GError
  *
@@ -230,9 +163,9 @@ g_mime_multipart_encrypted_new (void)
  **/
 int
 g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *content,
-				    GMimeCipherContext *ctx, gboolean sign,
-				    const char *userid, GPtrArray *recipients,
-				    GError **err)
+				    GMimeCryptoContext *ctx, gboolean sign,
+				    const char *userid, GMimeCryptoHash hash,
+				    GPtrArray *recipients, GError **err)
 {
 	GMimeStream *filtered_stream, *ciphertext, *stream;
 	GMimePart *version_part, *encrypted_part;
@@ -241,7 +174,7 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	GMimeFilter *crlf_filter;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_ENCRYPTED (mpe), -1);
-	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), -1);
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), -1);
 	g_return_val_if_fail (ctx->encrypt_protocol != NULL, -1);
 	g_return_val_if_fail (GMIME_IS_OBJECT (content), -1);
 	
@@ -262,7 +195,7 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	
 	/* encrypt the content stream */
 	ciphertext = g_mime_stream_mem_new ();
-	if (g_mime_cipher_context_encrypt (ctx, sign, userid, recipients, stream, ciphertext, err) == -1) {
+	if (g_mime_crypto_context_encrypt (ctx, sign, userid, hash, recipients, stream, ciphertext, err) == -1) {
 		g_object_unref (ciphertext);
 		g_object_unref (stream);
 		return -1;
@@ -315,7 +248,7 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 /**
  * g_mime_multipart_encrypted_decrypt:
  * @mpe: multipart/encrypted object
- * @ctx: decryption cipher context
+ * @ctx: decryption crypto context
  * @err: a #GError
  *
  * Attempts to decrypt the encrypted MIME part contained within the
@@ -330,7 +263,7 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
  * information as to why the failure occured.
  **/
 GMimeObject *
-g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherContext *ctx,
+g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCryptoContext *ctx,
 				    GError **err)
 {
 	GMimeObject *decrypted, *version, *encrypted;
@@ -345,7 +278,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	char *content_type;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_ENCRYPTED (mpe), NULL);
-	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), NULL);
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
 	g_return_val_if_fail (ctx->encrypt_protocol != NULL, NULL);
 	
 	if (mpe->decrypted) {
@@ -356,7 +289,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	protocol = g_mime_object_get_content_type_parameter (GMIME_OBJECT (mpe), "protocol");
 	
 	if (protocol) {
-		/* make sure the protocol matches the cipher encrypt protocol */
+		/* make sure the protocol matches the crypto encrypt protocol */
 		if (g_ascii_strcasecmp (ctx->encrypt_protocol, protocol) != 0) {
 			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
 				     "Cannot decrypt multipart/encrypted part: unsupported encryption protocol '%s'.",
@@ -374,8 +307,8 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	/* make sure the protocol matches the version part's content-type */
 	content_type = g_mime_content_type_to_string (version->content_type);
 	if (g_ascii_strcasecmp (content_type, protocol) != 0) {
-		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR, "%s",
-			     "Cannot decrypt multipart/encrypted part: content-type does not match protocol.");
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
+				     "Cannot decrypt multipart/encrypted part: content-type does not match protocol.");
 		
 		g_free (content_type);
 		
@@ -387,8 +320,8 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	encrypted = g_mime_multipart_get_part (GMIME_MULTIPART (mpe), GMIME_MULTIPART_ENCRYPTED_CONTENT);
 	mime_type = g_mime_object_get_content_type (encrypted);
 	if (!g_mime_content_type_is_type (mime_type, "application", "octet-stream")) {
-		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR, "%s",
-			     "Cannot decrypt multipart/encrypted part: unexpected content type");
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
+				     "Cannot decrypt multipart/encrypted part: unexpected content type");
 		
 		return NULL;
 	}
@@ -405,7 +338,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	g_object_unref (crlf_filter);
 	
 	/* get the cleartext */
-	if (!(sv = g_mime_cipher_context_decrypt (ctx, ciphertext, filtered_stream, err))) {
+	if (!(sv = g_mime_crypto_context_decrypt (ctx, ciphertext, filtered_stream, err))) {
 		g_object_unref (filtered_stream);
 		g_object_unref (stream);
 		
@@ -424,8 +357,8 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	g_object_unref (parser);
 	
 	if (!decrypted) {
-		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR, "%s",
-			     "Cannot decrypt multipart/encrypted part: failed to parse decrypted content");
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     "Cannot decrypt multipart/encrypted part: failed to parse decrypted content");
 		
 		g_mime_signature_validity_free (sv);
 		
