@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,30 +40,44 @@ extern int verbose;
 #define v(x) if (verbose > 3) x
 
 static gboolean
-request_passwd (GMimeCipherContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
+request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
 {
 	g_mime_stream_write_string (response, "no.secret\n");
 	
 	return TRUE;
 }
 
+static GMimeSignerStatus
+get_sig_status (GMimeSigner *signers)
+{
+	GMimeSignerStatus status = GMIME_SIGNER_STATUS_GOOD;
+	GMimeSigner *signer = signers;
+	
+	if (signers == NULL)
+		return GMIME_SIGNER_STATUS_ERROR;
+	
+	while (signer != NULL) {
+		status = MAX (status, signer->status);
+		signer = signer->next;
+	}
+	
+	return status;
+}
+
 static void
 print_verify_results (const GMimeSignatureValidity *validity)
 {
 	GMimeSigner *signer;
 	
-	switch (validity->status) {
-	case GMIME_SIGNATURE_STATUS_NONE:
-		fputs ("NONE\n", stdout);
-		break;
-	case GMIME_SIGNATURE_STATUS_GOOD:
+	switch (get_sig_status (validity->signers)) {
+	case GMIME_SIGNER_STATUS_GOOD:
 		fputs ("GOOD\n", stdout);
 		break;
-	case GMIME_SIGNATURE_STATUS_BAD:
+	case GMIME_SIGNER_STATUS_BAD:
 		fputs ("BAD\n", stdout);
 		break;
-	case GMIME_SIGNATURE_STATUS_UNKNOWN:
-		fputs ("Unknown status\n", stdout);
+	case GMIME_SIGNER_STATUS_ERROR:
+		fputs ("ERROR status\n", stdout);
 		break;
 	default:
 		fputs ("Unknown enum value\n", stdout);
@@ -84,6 +98,7 @@ print_verify_results (const GMimeSignatureValidity *validity)
 			break;
 		case GMIME_SIGNER_TRUST_NEVER:
 			fputs ("Never\n", stdout);
+			break;
 		case GMIME_SIGNER_TRUST_UNDEFINED:
 			fputs ("Undefined\n", stdout);
 			break;
@@ -100,9 +115,6 @@ print_verify_results (const GMimeSignatureValidity *validity)
 		
 		fprintf (stdout, "\tStatus: ");
 		switch (signer->status) {
-		case GMIME_SIGNER_STATUS_NONE:
-			fputs ("None\n", stdout);
-			break;
 		case GMIME_SIGNER_STATUS_GOOD:
 			fputs ("GOOD\n", stdout);
 			break;
@@ -151,7 +163,7 @@ that GMime properly treats MIME part content as opaque.\nIf this still verifies
 then we have ourselves a winner I guess...\n"
 
 static void
-test_multipart_signed (GMimeCipherContext *ctx)
+test_multipart_signed (GMimeCryptoContext *ctx)
 {
 	GMimeSignatureValidity *validity;
 	GMimeMultipartSigned *mps;
@@ -188,7 +200,7 @@ test_multipart_signed (GMimeCipherContext *ctx)
 	
 	/* sign the part */
 	g_mime_multipart_signed_sign (mps, GMIME_OBJECT (part), ctx, "no.user@no.domain",
-				      GMIME_CIPHER_HASH_SHA1, &err);
+				      GMIME_CRYPTO_HASH_SHA1, &err);
 	g_object_unref (part);
 	
 	if (err != NULL) {
@@ -246,7 +258,7 @@ test_multipart_signed (GMimeCipherContext *ctx)
 #define MULTIPART_ENCRYPTED_CONTENT "This is a test of multipart/encrypted.\n"
 
 static void
-test_multipart_encrypted (GMimeCipherContext *ctx, gboolean sign)
+test_multipart_encrypted (GMimeCryptoContext *ctx, gboolean sign)
 {
 	const GMimeSignatureValidity *sv;
 	GMimeStream *cleartext, *stream;
@@ -285,7 +297,8 @@ test_multipart_encrypted (GMimeCipherContext *ctx, gboolean sign)
 	recipients = g_ptr_array_new ();
 	g_ptr_array_add (recipients, "no.user@no.domain");
 	g_mime_multipart_encrypted_encrypt (mpe, GMIME_OBJECT (part), ctx, sign,
-					    "no.user@no.domain", recipients, &err);
+					    "no.user@no.domain", GMIME_CRYPTO_HASH_SHA256,
+					    recipients, &err);
 	g_ptr_array_free (recipients, TRUE);
 	g_object_unref (part);
 	
@@ -341,11 +354,11 @@ test_multipart_encrypted (GMimeCipherContext *ctx, gboolean sign)
 	v(print_verify_results (sv));
 	
 	if (sign) {
-		if (sv->status != GMIME_SIGNATURE_STATUS_GOOD)
-			ex = exception_new ("signature validity status expected to be GOOD");
+		if (get_sig_status (sv->signers) != GMIME_SIGNER_STATUS_GOOD)
+			ex = exception_new ("signature status expected to be GOOD");
 	} else {
-		if (sv->status != GMIME_SIGNATURE_STATUS_NONE)
-			ex = exception_new ("signature validity status expected to be NONE");
+		if (sv->signers != NULL)
+			ex = exception_new ("signature status expected to be NONE");
 	}
 	
 	if (ex != NULL) {
@@ -370,7 +383,7 @@ test_multipart_encrypted (GMimeCipherContext *ctx, gboolean sign)
 }
 
 static void
-import_key (GMimeCipherContext *ctx, const char *path)
+import_key (GMimeCryptoContext *ctx, const char *path)
 {
 	GMimeStream *stream;
 	GError *err = NULL;
@@ -381,7 +394,7 @@ import_key (GMimeCipherContext *ctx, const char *path)
 		throw (exception_new ("open() failed: %s", g_strerror (errno)));
 	
 	stream = g_mime_stream_fs_new (fd);
-	g_mime_cipher_context_import_keys (ctx, stream, &err);
+	g_mime_crypto_context_import_keys (ctx, stream, &err);
 	g_object_unref (stream);
 	
 	if (err != NULL) {
@@ -394,7 +407,7 @@ import_key (GMimeCipherContext *ctx, const char *path)
 int main (int argc, char *argv[])
 {
 	const char *datadir = "data/pgpmime";
-	GMimeCipherContext *ctx;
+	GMimeCryptoContext *ctx;
 	struct stat st;
 	char *key;
 	int i;
